
/* Copyright (c) 2012 by Al Williams

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial
portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

This software is intended to communicate with a GP9 device. Note that the above license covers,
However, this software does not include the firmware that resides inside the GP9. The above license
covers THIS software and not the GP9 firmware, which is not included in this distribution.

More info: http://www.awce.com/gp9.htm
al.williams@awce.com

*/

/** @file
 * This is the linux-specific gp3 file. Compile and link with
 * gp3lib.c. Do not link with other files (such as gp3cygwin.c). 
 * @author Al Williams al.williams@awce.com
 * @version 1.2
 * @date 4 Feb 2012
*/

using namespace std;

#include <QApplication>
#if defined(Q_OS_LINUX) || defined(Q_OS_UNIX)   // probably others too



// this was borrowed from GP3 (hence the names)
// and converted to CPP for no good reason
#include <cstdio>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <term.h>
#include "gp3linux.h"


/* Must operate at 9600 baud */
#define BAUDRATE B9600
#define DEBUGOUT 0
#if DEBUGOUT
#define DEBUG(tag,x) printf("DEBUG: %s=%02X\n",tag,x)
#else
#define DEBUG(tag,x)
#endif

/* This is the part that is platform specific */
/* This is for Linux or Cygwin */

static int fd;  // file descriptor
static struct termios oldtio; /* terminal info */


/* Open port at 9600 baud */
int gp3openport(const char *port)
{
  struct termios tio;
  int i;
  char lport[64]="/dev/";
  strcat(lport,port);
  // warning: putting O_NDELAY here causes reads to fail even with VTIME set
  // unless you put a silly (and speed-dependent) number of retries into read or block forever
  // however, some systems may block without DCD set
  // so the answer is to either make read block or retry a LARGE number of time
  // or set FNDELAY after the open (see below)
  // Since this is a Qt demo, I set O_NDELAY and infinite retries
  // BUT I yield to the Qt event loop while I retry so
  // it doesn't recover gracefully but it also doesn't hang
  fd=open(lport,O_RDWR|O_NOCTTY|O_NDELAY);
  if (fd<0)
    {
      return -1;
    }
//  fcntl(fd,F_SETFL,0);    // <<<<<---------<<<< Go back to blocking mode
  tcgetattr(fd,&oldtio); /* save old settings */
  memset(&tio,0,sizeof(tio));
  // Set up for 8N1, no processing */
  tio.c_cflag=BAUDRATE|CS8|CLOCAL|CREAD|CRTSCTS;
  tio.c_iflag=IGNPAR;
  tio.c_oflag=0;
  tio.c_lflag=0;
  for (i=0;i<NCCS;i++) tio.c_cc[i]=0;
  tio.c_cc[VMIN]=1;
  tio.c_cc[VTIME]=10;  // 1s timeout
  tcflush(fd,TCIFLUSH);
  tcsetattr(fd,TCSANOW,&tio); // make settings take  
  return 0;
}

/* Close the port when done */
void gp3closeport()
{
  tcsetattr(fd,TCSANOW,&oldtio);
  close(fd);
}

/* Write a byte to the GP3 */
int gp3write(int byte)
{
  char c=(char)byte;
  DEBUG("write",byte);
  return write(fd,&c,1);
}

/* Read a byte from the GP3 */
/* User may call this after repeat */
// if this hangs, it will block the UI unless you do comms in a seperate thread
// or you can set a reasonable # of retries
// but note that O_NDELAY on the open does not play nice with retries
unsigned int gp3read(void)
{
  char c;
  unsigned int rv;
  int ct;
  do {
    ct=read(fd,&c,1);
    if (ct==-1) QApplication::processEvents();
  } while (ct!=1);
// since c is char should be unnecessary
  rv=((unsigned int)c) & 0xFF;
  DEBUG("read",rv);
  return rv;
}

/* Check CTS (optional if you don't want to know if GP3 is busy.
   As written, the system will handle this automatically although
   your program will block until the GP3 is available. This will
   allow you to prevent that if you implement it.
   None of the portable code uses this!

   However, if you don't implement this, you'll need to remove
   the lines that call gp3ready from the example program.

   Not that your Linux/Unix or tty driver may not implement TIOCMGET
   or may not return correct results for this call. 
*/
int gp3ready(void)
{
  int s;
  usleep(100); // wait 100uS in case GP3 is just starting a command
  ioctl(fd,TIOCMGET,&s);
  return (s&TIOCM_CTS)==TIOCM_CTS;
}


#endif


